"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cosmiconfig_1 = __importDefault(require("cosmiconfig"));
const path_1 = __importDefault(require("path"));
const prettier_1 = __importDefault(require("prettier"));
const graphql_1 = require("graphql");
const type_graphql_1 = require("type-graphql");
exports.default = {
    name: 'generate',
    alias: ['g'],
    run: async (toolbox) => {
        const { config: { load }, parameters: { options, first, array }, print: { info, error }, string: { supplant }, template: { generate } } = toolbox;
        const config = load();
        const name = first;
        if (!name) {
            return error('name is required');
        }
        const names = {
            className: toolbox.strings.pascalCase(name),
            classNamePlural: toolbox.strings.pascalCase(toolbox.strings.plural(name)),
            camelName: toolbox.strings.camelCase(name),
            kebabName: toolbox.strings.kebabCase(name),
            camelNamePlural: toolbox.strings.camelCase(toolbox.strings.plural(name))
        };
        const cliGeneratePath = options.folder ||
            path_1.default.join(config.get('ROOT_FOLDER'), '/', config.get('CLI_GENERATE_PATH'), '/');
        const destFolder = supplant(cliGeneratePath, names);
        const tcgPathInGeneratedFolder = config.get('MODULE_IMPORT_PATH');
        const generatedPath = config.get('GENERATED_FOLDER');
        const generatedFolderRelativePath = path_1.default.relative(destFolder, generatedPath);
        let tcgPathInSourceFiles;
        if (tcgPathInGeneratedFolder === 'tcg') {
            tcgPathInSourceFiles = 'tcg';
        }
        else if (tcgPathInGeneratedFolder.charAt(0) === '@') {
            tcgPathInSourceFiles = tcgPathInGeneratedFolder;
        }
        else {
            const tcgAbsolutePath = path_1.default.join(generatedPath, tcgPathInGeneratedFolder);
            tcgPathInSourceFiles = path_1.default.relative(destFolder, tcgAbsolutePath);
        }
        const props = {
            ...names,
            pascalCase: toolbox.strings.pascalCase,
            camelCase: toolbox.strings.camelCase,
            kebabCase: toolbox.strings.kebabCase,
            fields: array ? processFields(array.slice(1)) : [],
            generatedFolderRelativePath,
            tcgPathInSourceFiles
        };
        await generateFile(toolbox, props, 'model.ts.ejs', destFolder, `${names.kebabName}.model.ts`);
        const target = path_1.default.join(destFolder, '/', `${names.kebabName}.service.ts`);
        await generate({
            template: 'service.ts.ejs',
            target,
            props
        });
        info(`Generated file at ${target}`);
        await generateFile(toolbox, props, 'resolver.ts.ejs', destFolder, `${names.kebabName}.resolver.ts`);
        return;
    }
};
async function generateFile(toolbox, props, template, destFolder, filename) {
    const target = path_1.default.join(destFolder, '/', filename);
    const explorer = cosmiconfig_1.default('prettier');
    const config = await explorer.search();
    let generated = await toolbox.template.generate({
        template,
        target,
        props
    });
    generated = prettier_1.default.format(generated, {
        ...(config ? config.config : {}),
        parser: 'typescript'
    });
    toolbox.filesystem.write(target, generated);
    toolbox.print.info(`Generated file at ${target}`);
}
function processFields(fields) {
    // If user doesn't pass fields, generate a single placeholder
    if (!fields.length) {
        fields = ['fieldName'];
    }
    return fields.map((raw) => {
        const match = raw.match(/^(?<name>[a-z-A-Z]+)($|:(?<type>[a-z-A-Z]+)((\/(?<relClass>[a-z-A-Z]+):(?<relField>[a-z-A-Z]+)\/)|)(?<required>!|))$/);
        if (!match || !match.groups) {
            throw new Error(`z`);
        }
        const { name, type, relClass, relField, required } = match.groups;
        let field = { name, gclType: graphql_1.GraphQLString, tsType: 'string' };
        field.required = Boolean(required);
        switch (type) {
            case 'id':
                field = { ...field, required: false, decorator: 'Id', tsType: 'IDType' };
                break;
            case 'createdAt':
                field = { ...field, required: false, decorator: 'CreatedAt', tsType: 'Date' };
                break;
            case 'updatedAt':
                field = { ...field, required: false, decorator: 'UpdatedAt', tsType: 'Date' };
                break;
            case 'boolean':
                field = { ...field, gclType: graphql_1.GraphQLBoolean, tsType: 'boolean' };
                break;
            case 'int':
                field = { ...field, gclType: graphql_1.GraphQLInt, tsType: 'number' };
                break;
            case 'float':
                field = { ...field, gclType: graphql_1.GraphQLFloat, tsType: 'number' };
                break;
            case 'date':
                field = { ...field, gclType: type_graphql_1.GraphQLISODateTime, tsType: 'Date' };
                break;
            case 'oto':
                field = {
                    ...field,
                    decorator: 'OneToOne',
                    gclType: relClass,
                    tsType: relClass,
                    relClass,
                    relField
                };
                break;
            case 'otm':
                field = {
                    ...field,
                    decorator: 'OneToMany',
                    gclType: relClass,
                    tsType: relClass,
                    relClass,
                    relField
                };
                break;
            case 'mto':
                field = {
                    ...field,
                    decorator: 'ManyToOne',
                    gclType: relClass,
                    tsType: relClass,
                    relClass,
                    relField
                };
                break;
            case 'mtm':
                field = {
                    ...field,
                    decorator: 'ManyToMany',
                    gclType: relClass,
                    tsType: relClass,
                    relClass,
                    relField
                };
                break;
        }
        return field;
    });
}
