"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const graphql_1 = require("graphql");
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = __importDefault(require("path"));
const type_graphql_1 = require("type-graphql");
const gql_1 = require("../../gql");
const schema_1 = require("../../schema");
const debug_1 = __importDefault(require("debug"));
const TypeORMConverter_1 = require("../../schema/TypeORMConverter");
const service_1 = require("../../schema/TypeORMConverter/service");
const debug = debug_1.default('tcg:code-generators');
class CodeGenerator {
    constructor(connection, generatedFolder, options) {
        this.connection = connection;
        this.generatedFolder = generatedFolder;
        this.options = options;
        if (!connection) {
            throw new Error('FileGenerator: connection is required');
        }
        this.createGeneratedFolder();
    }
    createGeneratedFolder() {
        mkdirp_1.default.sync(this.generatedFolder);
        mkdirp_1.default.sync(`${this.generatedFolder}`);
        mkdirp_1.default.sync(`${this.generatedFolder}/types`);
        mkdirp_1.default.sync(`${this.generatedFolder}/input`);
        mkdirp_1.default.sync(`${this.generatedFolder}/input/create`);
        mkdirp_1.default.sync(`${this.generatedFolder}/input/read`);
        mkdirp_1.default.sync(`${this.generatedFolder}/input/update`);
        mkdirp_1.default.sync(`${this.generatedFolder}/input/delete`);
        mkdirp_1.default.sync(`${this.generatedFolder}/resolvers`);
        mkdirp_1.default.sync(`${this.generatedFolder}/services`);
    }
    async generate() {
        debug('generate:start');
        await this.writeGeneratedIndexFile();
        await this.writeGeneratedTSTypes();
        await this.writeSchemaFile();
        await this.generateBinding();
        debug('generate:end');
    }
    async generateBinding() {
        debug('generateBinding:start');
        const schemaFilePath = path_1.default.join(this.generatedFolder, 'schema.graphql');
        const outputBindingPath = path_1.default.join(this.generatedFolder, 'binding.ts');
        const x = gql_1.generateBindingFile(schemaFilePath, outputBindingPath);
        debug('generateBinding:end');
        return x;
    }
    async buildGraphQLSchema() {
        if (!this.schema) {
            debug('buildGraphQLSchema:start');
            this.schema = await type_graphql_1.buildSchema({
                resolvers: this.options.resolversPath
            });
            debug('buildGraphQLSchema:end');
        }
        return this.schema;
    }
    async writeGeneratedTSTypes() {
        debug('writeGeneratedTSTypes:start');
        const generatedTSTypes = await this.getGeneratedTypes();
        const format = code => schema_1.SchemaGenerator.format(code);
        await Promise.all(this.connection.entityMetadatas
            .filter((entity) => entity.tableType !== 'junction')
            .map(async (entity) => {
            await this.writeToGeneratedClassesFolder(`${entity.name}.ts`, format(TypeORMConverter_1.entityToType(entity)), 'types');
            await this.writeToGeneratedClassesFolder(`${entity.name}.resolver.ts`, format(TypeORMConverter_1.entityToResolver(entity)), 'resolvers');
            await this.writeToGeneratedClassesFolder(`${entity.name}.service.ts`, format(service_1.entityToService(entity)), 'services');
            await this.writeToGeneratedClassesFolder(`${entity.name}WhereInput.ts`, format(TypeORMConverter_1.entityToWhereInput(entity)), 'input/read');
            await this.writeToGeneratedClassesFolder(`${entity.name}CreateInput.ts`, format(TypeORMConverter_1.entityToCreateInput(entity)), 'input/create');
            await this.writeToGeneratedClassesFolder(`${entity.name}ScalarWhereInput.ts`, format(TypeORMConverter_1.entityToWhereScalarInput(entity)), 'input/update');
            await this.writeToGeneratedClassesFolder(`${entity.name}UpdateInput.ts`, format(TypeORMConverter_1.entityToUpdateInput(entity)), 'input/update');
            await this.writeToGeneratedClassesFolder(`${entity.name}UpdateManyDataInput.ts`, format(TypeORMConverter_1.entityToUpdateDataInput(entity)), 'input/update');
            await this.writeToGeneratedClassesFolder(`${entity.name}UpdateManyWithWhereNestedInput.ts`, format(TypeORMConverter_1.entityToUpdateManyWithWhereNestedInput(entity)), 'input/update');
            const createInputs = TypeORMConverter_1.entityToCreateInputs(entity);
            await Promise.all(Object.keys(createInputs).map(async (createInput) => {
                await this.writeToGeneratedClassesFolder(`${createInput}.ts`, format(createInputs[createInput]), 'input/create');
            }));
            const updateInputs = TypeORMConverter_1.entityToUpdateInputs(entity);
            await Promise.all(Object.keys(updateInputs).map(async (updateInput) => {
                await this.writeToGeneratedClassesFolder(`${updateInput}.ts`, format(updateInputs[updateInput]), 'input/update');
            }));
        }));
        const x = this.writeToGeneratedClassesFolder('index.ts', generatedTSTypes);
        debug('writeGeneratedTSTypes:end');
        return x;
    }
    async getGeneratedTypes() {
        debug('getGeneratedTypes:start');
        const x = schema_1.SchemaGenerator.generate(this.connection.entityMetadatas, this.options.tcgImportPath);
        debug('getGeneratedTypes:end');
        return x;
    }
    async writeSchemaFile() {
        debug('writeSchemaFile:start');
        await this.buildGraphQLSchema();
        const schema = graphql_1.printSchema(this.schema);
        const x = this.writeToGeneratedFolder('schema.ts', `export default \`\n${schema.replace(/`/g, '\\`')}\` as any;`);
        const y = this.writeToGeneratedFolder('schema.graphql', schema);
        debug('writeSchemaFile:end');
        return Promise.all([x, y]);
    }
    async writeGeneratedIndexFile() {
        return this.writeToGeneratedFolder('index.ts', `export * from '.';`);
    }
    async writeToGeneratedFolder(filename, contents) {
        debug('writeToGeneratedFolder:' + filename + ':start');
        const x = fs_1.writeFileSync(path_1.default.join(this.generatedFolder, filename), contents, 'UTF-8');
        debug('writeToGeneratedFolder:' + filename + ':end');
        return x;
    }
    async writeToGeneratedClassesFolder(filename, contents, subFolder = undefined, rewrite = true) {
        debug('writeToGeneratedClassesFolder:' + filename + ':start');
        if (!rewrite) {
            const exist = await fs_1.existsSync(path_1.default.join(`${this.generatedFolder}${subFolder ? `/${subFolder}` : ''}`, filename));
            if (exist) {
                debug('writeToGeneratedClassesFolder:' + filename + ':skipped');
                return;
            }
        }
        const x = fs_1.writeFileSync(path_1.default.join(`${this.generatedFolder}${subFolder ? `/${subFolder}` : ''}`, filename), contents, 'UTF-8');
        debug('writeToGeneratedClassesFolder:' + filename + ':end');
        return x;
    }
}
exports.CodeGenerator = CodeGenerator;
