"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cosmiconfig_1 = __importDefault(require("cosmiconfig"));
const dotenv_1 = __importDefault(require("dotenv"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const typedi_1 = require("typedi");
const utils_1 = require("../utils");
const CONFIG_VALUE_VALID_KEYS = [
    'allowOptionalIdOnCreate',
    'generatedFolder',
    'cliGeneratePath',
    'moduleImportPath',
    'resolversPath'
];
let Config = class Config {
    constructor(options = {}) {
        this.options = options;
        this.TCG_ENV_PREFIX = 'TCG_';
        this.TYPEORM_ENV_PREFIX = 'TYPEORM_';
        this.TCG_DB_ENV_PREFIX = 'TCG_DB_';
        this.PROJECT_ROOT = process.cwd();
        this.container = options.container || typedi_1.Container;
        this.defaults = {
            TCG_DB_CONNECTION: 'postgres',
            TCG_ROOT_FOLDER: this.PROJECT_ROOT,
            TCG_ALLOW_OPTIONAL_ID_ON_CREATE: 'false',
            TCG_APP_PROTOCOL: 'https',
            TCG_AUTO_GENERATE_FILES: 'false',
            TCG_AUTO_OPEN_PLAYGROUND: 'false',
            TCG_INTROSPECTION: 'true',
            TCG_CLI_GENERATE_PATH: './src',
            TCG_DB_ENTITIES: ['src/**/*.model.ts'],
            TCG_DB_ENTITIES_DIR: 'src/models',
            TCG_DB_LOGGER: 'advanced-console',
            TCG_DB_MIGRATIONS: ['db/migrations/**/*.ts'],
            TCG_DB_MIGRATIONS_DIR: 'db/migrations',
            TCG_DB_PORT: 5432,
            TCG_DB_SUBSCRIBERS: ['src/subscribers/**/*.ts'],
            TCG_DB_SUBSCRIBERS_DIR: 'src/subscribers',
            TCG_DB_SYNCHRONIZE: 'false',
            TCG_MODULE_IMPORT_PATH: 'tcg',
            // TODO: eventually we should do this path resolution when we ask for the variable with `get`
            TCG_GENERATED_FOLDER: path_1.default.join(this.PROJECT_ROOT, 'generated'),
            TCG_RESOLVERS_PATH: [path_1.default.join(this.PROJECT_ROOT, 'src/**/*.resolver.ts')]
        };
        this.devDefaults = {
            TCG_APP_HOST: 'localhost',
            TCG_APP_PORT: '4000',
            TCG_APP_PROTOCOL: 'http',
            TCG_AUTO_GENERATE_FILES: 'true',
            TCG_AUTO_OPEN_PLAYGROUND: 'true',
            TCG_DB_HOST: 'localhost',
            TCG_DB_LOGGING: 'all'
        };
        const dotenvPath = options.dotenvPath || this.PROJECT_ROOT;
        this.NODE_ENV = this.determineNodeEnv(dotenvPath);
        this.loadDotenvFiles(dotenvPath);
        return this.loadSync();
    }
    // Allow NODE_ENV to be set in the .env file.  Check for this first here and then fall back on
    // the environment variable.  The reason we do this is because using dotenvi will allow us to switch
    // between environments.  If we require an actual environment variable to be set then we'll have to set
    // and unset the value in the current terminal buffer.
    determineNodeEnv(dotenvPath) {
        let nodeEnv = process.env.NODE_ENV;
        const filepath = path_1.default.join(dotenvPath, '.env');
        if (fs_1.default.existsSync(filepath)) {
            const config = dotenv_1.default.parse(fs_1.default.readFileSync(filepath));
            if (config.NODE_ENV) {
                nodeEnv = config.NODE_ENV;
            }
        }
        return (this.NODE_ENV = process.env.NODE_ENV = nodeEnv);
    }
    loadDotenvFiles(dotenvPath) {
        // .local files are for secrets, load those first
        const files = [`.env.local.${this.NODE_ENV}`, '.env.local', '.env'];
        files.forEach((filename) => {
            const filepath = path_1.default.join(dotenvPath, filename);
            if (fs_1.default.existsSync(filepath)) {
                dotenv_1.default.config({ path: filepath });
            }
        });
    }
    loadSync() {
        const devOptions = this.NODE_ENV === 'development' ? this.devDefaults : {};
        const configFile = this.loadStaticConfigSync();
        // Config is loaded as a waterfall.  Items at the top of the object are overwritten by those below, so the order is:
        // - Add application-wide defaults
        // - Add development defaults (if we're runnign in DEV mode)
        // - Load config from config file
        // - Load environment variables
        // - Override with locked options
        const combined = {
            ...this.defaults,
            ...devOptions,
            ...configFile,
            ...this.typeORMToTcgEnvVariables(),
            ...this.tcgEnvVariables()
        };
        // If Jest is running, be smart and don't open playground
        if (typeof process.env.JEST_WORKER_ID !== 'undefined') {
            combined.TCG_AUTO_OPEN_PLAYGROUND = 'false';
        }
        // Make sure to set the DB connection if we're using a mock DB
        if (combined['TCG_MOCK_DATABASE'] === 'true') {
            combined['TCG_DB_CONNECTION'] = 'sqlite';
            combined['TCG_DB_DATABASE'] = 'tcg.sqlite.tmp';
            combined['TCG_DB_SYNCHRONIZE'] = 'true';
        }
        this.config = combined;
        // Must be after config is set above
        this.validateEntryExists('TCG_APP_HOST');
        this.validateEntryExists('TCG_APP_PORT');
        this.validateEntryExists('TCG_GENERATED_FOLDER');
        this.validateEntryExists('TCG_DB_CONNECTION');
        this.validateEntryExists('TCG_DB_HOST');
        // Now that we've pulled all config in from the waterfall, write `TCG_DB_` keys to `TYPEORM_`
        // So that TypeORM will pick them up
        // this.writeTcgConfigToTypeORMEnvVars();
        // Once we've combined all of the Tcg ENV vars, write them to process.env so that they can be used elsewhere
        // NOTE: this is likely a bad idea and we should use Containers
        this.writeTcgEnvVars();
        // console.log('loadSync complete', this.get());
        return this;
    }
    get(key) {
        if (!key) {
            return this.config;
        }
        const lookup = key.startsWith(this.TCG_ENV_PREFIX) ? key : `${this.TCG_ENV_PREFIX}${key}`;
        return this.config[lookup];
    }
    tcgEnvVariables() {
        return this.envVarsByPrefix(this.TCG_ENV_PREFIX);
    }
    tcgDBEnvVariables() {
        return this.envVarsByPrefix(this.TCG_DB_ENV_PREFIX);
    }
    typeORMEnvVariables() {
        return this.envVarsByPrefix(this.TYPEORM_ENV_PREFIX);
    }
    translateEnvVar(key, value) {
        const arrayTypes = [
            'TCG_DB_ENTITIES',
            'TCG_DB_MIGRATIONS',
            'TCG_DB_SUBSCRIBERS',
            'TCG_RESOLVERS_PATH'
        ];
        const pathTypes = ['TCG_GENERATED_FOLDER'];
        // Should be able to do this, but TypeGraphQL has an issue with relative requires
        // https://github.com/19majkel94/type-graphql/blob/a212fd19f28d3095244c44381617f03e97ec4db3/src/helpers/loadResolversFromGlob.ts#L4
        // const paths = value.split(',');
        if (arrayTypes.indexOf(key) > -1) {
            return value.split(',').map((item) => {
                if (path_1.default.isAbsolute(item)) {
                    return item;
                }
                return path_1.default.join(this.PROJECT_ROOT, item);
            });
        }
        if (pathTypes.indexOf(key) > -1) {
            if (path_1.default.isAbsolute(value)) {
                return value;
            }
            return path_1.default.join(this.PROJECT_ROOT, value);
        }
        return value;
    }
    envVarsByPrefix(prefix) {
        const config = {};
        Object.keys(process.env).forEach((key) => {
            if (key.startsWith(prefix)) {
                config[key] = this.translateEnvVar(key, process.env[key] || '');
            }
        });
        return config;
    }
    typeORMToTcgEnvVariables() {
        const typeORMvars = this.typeORMEnvVariables();
        const config = {};
        Object.keys(typeORMvars).forEach((key) => {
            const keySuffix = key.substring(this.TYPEORM_ENV_PREFIX.length);
            config[`${this.TCG_DB_ENV_PREFIX}${keySuffix}`] = typeORMvars[key];
        });
        return config;
    }
    // public writeTcgConfigToTypeORMEnvVars() {
    //   Object.keys(this.config).forEach((key: string) => {
    //     if (key.startsWith(this.TCG_DB_ENV_PREFIX)) {
    //       const keySuffix = key.substring(this.TCG_DB_ENV_PREFIX.length);
    //       process.env[`TYPEORM_${keySuffix}`] = this.get(key);
    //     }
    //   });
    // }
    writeTcgEnvVars() {
        Object.keys(this.config).forEach((key) => {
            if (key.startsWith(this.TCG_ENV_PREFIX)) {
                process.env[key] = this.get(key);
            }
        });
    }
    validateEntryExists(key) {
        if (!this.config) {
            throw new Error("Can't validate the base config until after it is generated");
        }
        const value = this.get(key);
        if (!value) {
            throw new Error(`Config: ${key} is required: ${value}\n\n${JSON.stringify(this.config)}\n\n${JSON.stringify(process.env)}`);
        }
    }
    loadStaticConfigSync() {
        const response = this.loadStaticConfigFileSync();
        if (typeof response === 'undefined') {
            return {};
        }
        const constantized = utils_1.ObjectUtil.constantizeKeys(response.config);
        return utils_1.ObjectUtil.prefixKeys(constantized, this.TCG_ENV_PREFIX);
    }
    // Use cosmiconfig to load static config that has to be the same for all environments
    // paths to folders for the most part
    loadStaticConfigFileSync() {
        const explorer = cosmiconfig_1.default('tcg');
        // Pull config values from cosmiconfig
        const results = explorer.searchSync(this.options.configSearchPath);
        if (!results || results.isEmpty) {
            return undefined;
        }
        const userConfigKeys = Object.keys(results.config);
        const badKeys = userConfigKeys.filter(x => !CONFIG_VALUE_VALID_KEYS.includes(x));
        if (badKeys.length) {
            throw new Error(`Config: invalid keys specified in ${results.filepath}: [${badKeys.join(', ')}]`);
        }
        // Make sure the generated folder is an absolute path
        if (results.config.generatedFolder && !path_1.default.isAbsolute(results.config.generatedFolder)) {
            results.config.generatedFolder = path_1.default.join(path_1.default.dirname(results.filepath), results.config.generatedFolder);
        }
        return results;
    }
};
Config = __decorate([
    typedi_1.Service('Config'),
    __metadata("design:paramtypes", [Object])
], Config);
exports.Config = Config;
