"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const filter_1 = require("../../sorm/filter");
const metadata_1 = require("../../metadata");
const graphql_fields_1 = __importDefault(require("graphql-fields"));
class MainService {
    constructor(entityClass, connection) {
        this.entityClass = entityClass;
        this.connection = connection;
        if (!entityClass) {
            throw new Error('BaseResolver requires an entity Class');
        }
        this.repository = connection.getRepository(entityClass);
        if (!this.repository) {
            throw new Error(`BaseResolver requires a valid repository, class ${entityClass}`);
        }
        if (!this.pk) {
            this.pk =
                metadata_1.getMetadataStorage().getPk(entityClass.name) ||
                    (() => {
                        for (const column of (this.repository.metadata || this.repository.getMetadata()).primaryColumns) {
                            if (column.target === (this.repository.metadata || this.repository.getMetadata()).target ||
                                typeof column.target !== 'function') {
                                continue;
                            }
                            const pk = metadata_1.getMetadataStorage().getPk(column.target.name);
                            if (pk) {
                                return pk;
                            }
                        }
                        throw new Error(`${entityClass} must have decorator Id for Model.`);
                    })();
        }
    }
    async findOne(where, fields) {
        const query = filter_1.filterQuery(this.entityClass, { where });
        fields = (fields || []).filter(field => (this.repository.metadata || this.repository.getMetadata()).columns.some(column => column.propertyName === field));
        if (fields.length) {
            if (!fields.includes(this.pk)) {
                fields.push(this.pk);
            }
            query.select(fields.map(field => `${query.alias}.${field}`));
        }
        return query.getOne();
    }
    async findMany(args, fields) {
        const query = filter_1.filterQuery(this.entityClass, args);
        fields = (fields || []).filter(field => (this.repository.metadata || this.repository.getMetadata()).columns.some(column => column.propertyName === field));
        if (filter_1.getCursor(this.entityClass, query, args, this.pk).isBackward) {
            return Promise.resolve((await query.getMany()).reverse());
        }
        if (fields.length) {
            if (!fields.includes(this.pk)) {
                fields.push(this.pk);
            }
            query.select(fields.map(field => `${query.alias}.${field}`));
        }
        return query.getMany();
    }
    async findManyConnection(args, info) {
        const rawFields = graphql_fields_1.default(info);
        const rawFieldsNode = ('edges' in rawFields && 'node' in rawFields.edges && rawFields.edges.node) || {};
        const fields = Object.keys(rawFieldsNode).filter(item => {
            return (Object.keys(rawFieldsNode[item]).length === 0 &&
                (this.repository.metadata || this.repository.getMetadata()).columns.some(column => column.propertyName === item));
        });
        const query = filter_1.filterQuery(this.entityClass, args);
        /*const relationsFields = (this.repository.metadata||(this.repository as any).getMetadata()).columns
          .filter((column: ColumnMetadata) => !column.relationMetadata)
          .map((column: ColumnMetadata) => column.propertyName);*/
        if (fields.length) {
            if (!fields.includes(this.pk)) {
                fields.push(this.pk);
            }
            query.select(fields.map(field => `${query.alias}.${field}`));
        }
        const { slice, isBackward } = filter_1.getCursor(this.entityClass, query, args, this.pk, true);
        const entities = await query.getMany();
        const edges = isBackward
            ? entities.slice(0, slice || undefined).reverse()
            : entities.slice(0, slice || undefined);
        let startCursor, endCursor, hasNextPage = false, hasPreviousPage = false;
        const count = edges.length;
        if (count) {
            startCursor = edges[0][this.pk];
            endCursor = edges[count - 1][this.pk];
            hasNextPage = (!isBackward && entities.length > count) || false;
            hasPreviousPage = (isBackward && entities.length > count) || false;
        }
        return {
            edges: (edges => edges.map((node) => ({
                cursor: node[this.pk],
                node
            })))(edges),
            pageInfo: {
                hasNextPage,
                hasPreviousPage,
                startCursor,
                endCursor
            },
            aggregate: {
                count
            }
        };
    }
    async create(data) {
        data = Object.fromEntries(Object.entries(data).filter(([_key, value]) => value !== undefined));
        const queryRunner = await this.repository.manager.connection.createQueryRunner();
        if (!queryRunner)
            throw new Error(`queryRunner doesn't init.`);
        const entity = Object.assign(new this.entityClass(), data);
        const processCreateRelations = (relation, data) => {
            if (typeof relation.inverseEntityMetadata.target !== 'function')
                throw new Error(`Relation target is not function.`);
            switch (relation.relationType) {
                case 'one-to-one':
                case 'many-to-one':
                    return processRelations(Object.assign(new relation.inverseEntityMetadata.target(), data));
                case 'one-to-many':
                case 'many-to-many':
                    return data.map(value => processRelations(Object.assign(new relation.inverseEntityMetadata.target(), value)));
            }
        };
        const processConnectRelations = (relation, data) => {
            if (typeof relation.inverseEntityMetadata.target !== 'function')
                throw new Error(`Relation target is not function.`);
            switch (relation.relationType) {
                case 'one-to-one':
                case 'many-to-one':
                    return Object.assign(new relation.inverseEntityMetadata.target(), data);
                case 'one-to-many':
                case 'many-to-many':
                    return data.map(value => Object.assign(new relation.inverseEntityMetadata.target(), value));
            }
        };
        function processRelations(data) {
            const meta = queryRunner.manager.getRepository(data.constructor).metadata || queryRunner.manager.getRepository(data.constructor).getMetadata();
            const relations = meta.relations.map(relation => relation.propertyName);
            Object.keys(data).forEach((key) => {
                const index = relations.indexOf(key);
                if (index === -1)
                    return;
                const relation = meta.relations[index];
                switch (relation.relationType) {
                    case 'one-to-one':
                    case 'many-to-one':
                        data[key] =
                            'create' in data[key]
                                ? processCreateRelations(relation, data[key].create)
                                : 'connect' in data[key]
                                    ? processConnectRelations(relation, data[key].connect)
                                    : {};
                        break;
                    case 'one-to-many':
                    case 'many-to-many':
                        data[key] = [
                            ...('create' in data[key]
                                ? (value => (Array.isArray(value) ? value : []))(processCreateRelations(relation, data[key].create))
                                : []),
                            ...('connect' in data[key]
                                ? (value => (Array.isArray(value) ? value : []))(processConnectRelations(relation, data[key].connect))
                                : [])
                        ];
                        break;
                }
            });
            return data;
        }
        try {
            await queryRunner.startTransaction();
            processRelations(entity);
            await queryRunner.manager.save(entity);
            await queryRunner.commitTransaction();
        }
        catch (err) {
            await queryRunner.rollbackTransaction();
            throw new Error(err.detail ? err.detail : err.message);
        }
        finally {
            await queryRunner.release();
        }
        return entity;
    }
    async update(data, where) {
        data = Object.fromEntries(Object.entries(data).filter(([_key, value]) => value !== undefined));
        const queryRunner = await this.repository.manager.connection.createQueryRunner();
        if (!queryRunner)
            throw new Error(`queryRunner doesn't init.`);
        const entity = await queryRunner.manager.findOneOrFail(this.entityClass, {
            where: { ...where }
        });
        const processCreateRelations = async (entityMeta, data, key, of) => {
            const meta = entityMeta.relations.find((relation) => relation.propertyName === key);
            if (!meta || !meta.inverseEntityMetadata || !meta.inverseEntityMetadata.target) {
                return data;
            }
            const invertedRelations = meta.inverseEntityMetadata.relations.map(relation => relation.propertyName);
            let inserted;
            switch (meta.relationType) {
                case 'one-to-one':
                case 'many-to-one':
                    inserted = await queryRunner.connection.manager.insert(meta.inverseEntityMetadata.target, Object.assign(new meta.inverseEntityMetadata.target(), Object.fromEntries(Object.entries(data).filter(([key]) => !invertedRelations.includes(key)))));
                    queryRunner.connection
                        .createQueryBuilder(queryRunner)
                        .relation(entityMeta.target, key)
                        .of(of)
                        .set([...inserted.identifiers].pop());
                    await Promise.all(inserted.identifiers.map(async (identifier) => {
                        await processRelations(Object.assign(new meta.inverseEntityMetadata.target(), data), Object.assign(new meta.inverseEntityMetadata.target(), identifier));
                    }));
                    break;
                case 'one-to-many':
                case 'many-to-many':
                    inserted = await queryRunner.connection.manager.insert(meta.inverseEntityMetadata.target, await Promise.all(data.map(async (dt) => Object.assign(new meta.inverseEntityMetadata.target(), Object.fromEntries(Object.entries(dt).filter(([key]) => !invertedRelations.includes(key)))))));
                    queryRunner.connection
                        .createQueryBuilder(queryRunner)
                        .relation(entityMeta.target, key)
                        .of(of)
                        .add(inserted.identifiers);
                    await Promise.all(inserted.identifiers.map(async (identifier, index) => {
                        await processRelations(Object.assign(new meta.inverseEntityMetadata.target(), data[index]), Object.assign(new meta.inverseEntityMetadata.target(), identifier));
                    }));
                    break;
            }
        };
        const processConnectAndDisconnectRelations = async (entityMeta, connect, disconnect, set, key, of) => {
            const meta = entityMeta.relations.find((relation) => relation.propertyName === key);
            if (!meta || !meta.inverseEntityMetadata || !meta.inverseEntityMetadata.target || !of) {
                return;
            }
            switch (meta.relationType) {
                case 'one-to-one':
                case 'many-to-one':
                    if (disconnect) {
                        queryRunner.connection
                            .createQueryBuilder(queryRunner)
                            .relation(entityMeta.target, key)
                            .of(of)
                            .set(null);
                    }
                    else if (connect) {
                        queryRunner.connection
                            .createQueryBuilder(queryRunner)
                            .relation(entityMeta.target, key)
                            .of(of)
                            .set(Object.assign(meta.inverseEntityMetadata.target, connect));
                    }
                    break;
                case 'one-to-many':
                case 'many-to-many':
                    if (Array.isArray(set) && set.length) {
                        queryRunner.connection
                            .createQueryBuilder(queryRunner)
                            .relation(entityMeta.target, key)
                            .of(of)
                            .set((set || []).map(st => Object.assign(meta.inverseEntityMetadata.target, st)));
                    }
                    else {
                        queryRunner.connection
                            .createQueryBuilder(queryRunner)
                            .relation(entityMeta.target, key)
                            .of(of)
                            .remove((disconnect || []).map(dt => Object.assign(meta.inverseEntityMetadata.target, dt)));
                        queryRunner.connection
                            .createQueryBuilder(queryRunner)
                            .relation(entityMeta.target, key)
                            .of(of)
                            .add((connect || []).map(ct => Object.assign(meta.inverseEntityMetadata.target, ct)));
                    }
                    break;
            }
        };
        const processDeleteRelations = async (entityMeta, del, key) => {
            const meta = entityMeta.relations.find((relation) => relation.propertyName === key);
            if (!meta || !meta.inverseEntityMetadata || !meta.inverseEntityMetadata.target) {
                return del;
            }
            switch (meta.relationType) {
                case 'one-to-one':
                case 'many-to-one':
                    if (!del)
                        break;
                    await queryRunner.manager.remove(Object.assign(new meta.inverseEntityMetadata.target(), del));
                    break;
                case 'one-to-many':
                case 'many-to-many':
                    if (!del.length)
                        break;
                    await queryRunner.manager.remove(del.map(dl => Object.assign(new meta.inverseEntityMetadata.target(), dl)));
                    break;
            }
        };
        async function processRelations(data, of) {
            const meta = queryRunner.manager.getRepository(data.constructor).metadata || queryRunner.manager.getRepository(data.constructor).getMetadata();
            const relations = meta.relations.map(relation => relation.propertyName);
            await Promise.all(Object.keys(data).map(async (key) => {
                if (relations.includes(key)) {
                    if ('create' in data[key]) {
                        await processCreateRelations(meta, data[key].create, key, of);
                    }
                    if ('connect' in data[key] || 'disconnect' in data[key] || 'set' in data[key]) {
                        await processConnectAndDisconnectRelations(meta, data[key].connect, data[key].disconnect, data[key].set, key, of);
                    }
                    if ('delete' in data[key]) {
                        await processDeleteRelations(meta, data[key].delete, key);
                    }
                    if ('update' in data[key]) {
                        throw new Error(`Method update doesn't implemented yet.`);
                    }
                    else if ('updateMany' in data[key]) {
                        throw new Error(`Method updateMany doesn't implemented yet.`);
                    }
                    else if ('deleteMany' in data[key]) {
                        throw new Error(`Method deleteMany doesn't implemented yet.`);
                    }
                }
            }));
        }
        try {
            await queryRunner.startTransaction();
            await processRelations(Object.assign(new this.entityClass(), data), entity);
            const invertedRelations = (this.repository.metadata || this.repository.getMetadata()).relations.map(relation => relation.propertyName);
            await queryRunner.manager.save(Object.assign(entity, Object.fromEntries(Object.entries(data).filter(([key]) => !invertedRelations.includes(key)))));
            await queryRunner.commitTransaction();
        }
        catch (err) {
            await queryRunner.rollbackTransaction();
            throw new Error(err.message);
        }
        finally {
            await queryRunner.release();
        }
        return entity;
    }
    async updateMany(_data, _where) {
        const queryRunner = await this.repository.manager.connection.createQueryRunner();
        if (!queryRunner)
            throw new Error(`queryRunner doesn't init.`);
        // const entity: Entity = Object.assign(new this.entityClass(), data);
        throw new Error(`Method updateMany doesn't implemented yet.`);
        // return entity;
    }
    async delete(where) {
        const queryRunner = await this.repository.manager.connection.createQueryRunner();
        if (!queryRunner)
            throw new Error(`queryRunner doesn't init.`);
        const entity = await queryRunner.manager.findOneOrFail(this.entityClass, {
            where: {
                ...where
            }
        });
        try {
            await queryRunner.startTransaction();
            await queryRunner.manager.remove(entity);
            await queryRunner.commitTransaction();
        }
        catch (err) {
            await queryRunner.rollbackTransaction();
            throw new Error(err.message);
        }
        finally {
            await queryRunner.release();
        }
        return true;
    }
    async deleteMany(where) {
        const queryRunner = await this.repository.manager.connection.createQueryRunner();
        if (!queryRunner)
            throw new Error(`queryRunner doesn't init.`);
        const entities = await this.findMany({ where });
        let count = entities.length;
        try {
            await queryRunner.startTransaction();
            count = (await queryRunner.manager.remove(entities)).length || count;
            await queryRunner.commitTransaction();
        }
        catch (err) {
            await queryRunner.rollbackTransaction();
            throw new Error(err.message);
        }
        finally {
            await queryRunner.release();
        }
        return { count };
    }
}
exports.default = MainService;
