"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const metadata_1 = require("../../metadata");
const graphql_type_json_1 = require("graphql-type-json");
const type_graphql_1 = require("type-graphql");
exports.uniquesForEntity = (entity) => {
    return entity.uniques.reduce((arr, unique) => [
        ...arr,
        ...unique.columns
            .filter((col) => !col.relationMetadata)
            .map((col) => col.propertyName)
    ], []);
};
function filenameToImportPath(filename) {
    return filename.replace(/\.([jt])s$/, '').replace(/\\/g, '/');
}
exports.filenameToImportPath = filenameToImportPath;
function columnToTypes(column) {
    const graphqlType = columnToGraphQLType(column);
    const tsType = columnToTypeScriptType(column);
    return { graphqlType, tsType };
}
exports.columnToTypes = columnToTypes;
function relationToTypes(relation) {
    const graphqlType = relationToGraphQLType(relation);
    const tsType = relationToTypeScriptType(relation);
    return { graphqlType, tsType };
}
exports.relationToTypes = relationToTypes;
function inverseRelationToTypes(inverseRelation) {
    const inverseGraphqlType = inverseRelationToGraphQLType(inverseRelation);
    const inverseTsType = inverseRelationToTypeScriptType(inverseRelation);
    return { inverseGraphqlType, inverseTsType };
}
exports.inverseRelationToTypes = inverseRelationToTypes;
function extractEnumObject(column) {
    return metadata_1.getMetadataStorage().getEnum(column.entityMetadata.name, column.propertyName);
}
exports.extractEnumObject = extractEnumObject;
function columnToTypeScriptType(column) {
    if (column.isPrimary) {
        return 'IDType';
    }
    else if (column.enum) {
        return column.isArray ? `${extractEnumObject(column).name}[]` : extractEnumObject(column).name;
    }
    else {
        const graphqlType = columnTypeToGraphQLDataType(column);
        const typeMap = {
            GraphQLBoolean: 'boolean',
            GraphQLISODateTime: 'Date',
            GraphQLFloat: 'number',
            GraphQLJSONObject: 'JSON',
            GraphQLID: 'IDType',
            GraphQLInt: 'number',
            GraphQLString: 'string'
        };
        return typeMap[graphqlType] || 'string';
    }
}
exports.columnToTypeScriptType = columnToTypeScriptType;
function relationToTypeScriptType(relation) {
    const graphqlType = relationTypeToGraphQLDataType(relation);
    switch (graphqlType) {
        default:
            return graphqlType;
    }
}
exports.relationToTypeScriptType = relationToTypeScriptType;
function inverseRelationToTypeScriptType(inverseRelation) {
    const graphqlType = inverseRelationTypeToGraphQLDataType(inverseRelation);
    switch (graphqlType) {
        default:
            return graphqlType;
    }
}
exports.inverseRelationToTypeScriptType = inverseRelationToTypeScriptType;
function columnTypeToGraphQLDataType(column) {
    const graphQLType = columnToGraphQLType(column);
    if (graphQLType instanceof graphql_1.GraphQLList) {
        switch (graphQLType.ofType) {
            case graphql_1.GraphQLID:
                return '[GraphQLID]';
            case graphql_1.GraphQLString:
                return '[GraphQLString]';
            case graphql_1.GraphQLBoolean:
                return '[GraphQLBoolean]';
            case graphql_1.GraphQLFloat:
                return '[GraphQLFloat]';
            case graphql_1.GraphQLInt:
                return '[GraphQLInt]';
            case type_graphql_1.GraphQLISODateTime:
                return '[GraphQLISODateTime]';
            case graphql_type_json_1.GraphQLJSONObject:
                return '[GraphQLJSONObject]';
            default:
                return String(graphQLType);
        }
    }
    switch (graphQLType) {
        case graphql_1.GraphQLID:
            return 'GraphQLID';
        case graphql_1.GraphQLString:
            return 'GraphQLString';
        case graphql_1.GraphQLBoolean:
            return 'GraphQLBoolean';
        case graphql_1.GraphQLFloat:
            return 'GraphQLFloat';
        case graphql_1.GraphQLInt:
            return 'GraphQLInt';
        case type_graphql_1.GraphQLISODateTime:
            return 'GraphQLISODateTime';
        case graphql_type_json_1.GraphQLJSONObject:
            return 'GraphQLJSONObject';
        default:
            return graphQLType.name;
    }
}
exports.columnTypeToGraphQLDataType = columnTypeToGraphQLDataType;
function relationTypeToGraphQLDataType(relation) {
    const graphQLType = relationToGraphQLType(relation);
    switch (graphQLType) {
        default:
            return graphQLType;
    }
}
exports.relationTypeToGraphQLDataType = relationTypeToGraphQLDataType;
function inverseRelationTypeToGraphQLDataType(inverseRelation) {
    const graphQLType = inverseRelationToGraphQLType(inverseRelation);
    switch (graphQLType) {
        default:
            return graphQLType;
    }
}
exports.inverseRelationTypeToGraphQLDataType = inverseRelationTypeToGraphQLDataType;
function columnToGraphQLType(column) {
    const enumType = extractEnumObject(column);
    if (column.enum) {
        return column.isArray ? graphql_1.GraphQLList(new graphql_1.GraphQLEnumType({
            name: enumType.name,
            values: Object.fromEntries(Object.entries(enumType.enumeration).map(([key, value]) => [key, { value }]))
        })) : enumType;
    }
    else if (column.isPrimary) {
        return graphql_1.GraphQLID;
    }
    const type = column.type.name ? column.type.name : column.type;
    switch (type) {
        case 'simple-array':
            return graphql_1.GraphQLList(graphql_1.GraphQLString);
        case String:
        case 'String':
        case 'varchar':
        case 'text':
        case 'uuid':
            return graphql_1.GraphQLString;
        case Boolean:
        case 'Boolean':
        case 'boolean':
        case 'bool':
            return graphql_1.GraphQLBoolean;
        case Number:
        case 'Number':
        case 'float4':
        case 'float8':
        case 'smallmoney':
        case 'money':
        case 'double':
        case 'dec':
        case 'decimal':
        case 'fixed':
        case 'numeric':
        case 'real':
        case 'double precision':
            return graphql_1.GraphQLFloat;
        case 'int':
        case 'smallint':
        case 'mediumint':
        case 'bigint':
        case 'integer':
        case 'int2':
        case 'int4':
        case 'int8':
            return graphql_1.GraphQLInt;
        case Date:
        case 'Date':
        case 'timestamp':
        case 'timestamp without time zone':
            return type_graphql_1.GraphQLISODateTime;
        case 'json':
        case 'jsonb':
        case 'simple-json':
            return graphql_type_json_1.GraphQLJSONObject;
        default:
            if (type instanceof graphql_1.GraphQLScalarType) {
                return type;
            }
            throw new Error(`convertToGraphQLType: Unexpected type: ${type}`);
    }
}
exports.columnToGraphQLType = columnToGraphQLType;
function relationToGraphQLType(relation) {
    return Object.assign(graphql_1.GraphQLObjectType.prototype, {
        name: relation.inverseEntityMetadata.name
    });
    // throw new Error(`convertToGraphQLRelationType: Unexpected type: ${relation.type}`);
}
exports.relationToGraphQLType = relationToGraphQLType;
function inverseRelationToGraphQLType(inverseRelation) {
    return Object.assign(graphql_1.GraphQLObjectType.prototype, {
        name: inverseRelation.inverseEntityMetadata.name
    });
    // throw new Error(`convertToGraphQLRelationType: Unexpected type: ${relation.type}`);
}
exports.inverseRelationToGraphQLType = inverseRelationToGraphQLType;
