"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeorm_1 = require("typeorm");
const FindCriteriaNotFoundError_1 = require("typeorm/error/FindCriteriaNotFoundError");
const metadata_1 = require("../../metadata");
const types_1 = require("./types");
const operators_1 = require("./operators");
const buildWhere = (query, key, fo, embedPrefix, root, subBuilder) => {
    let parameterIndex = Object.keys((root || query).expressionMap.nativeParameters).length;
    const propertyPath = embedPrefix ? embedPrefix + '.' + key : key;
    const mainAlias = (subBuilder || query).expressionMap.mainAlias;
    if (!mainAlias)
        throw new Error(`Can't get mainAlias (filterQuery)`);
    const metadata = mainAlias.metadata || mainAlias.getMetadata();
    const alias = mainAlias.name;
    const column = metadata.findColumnWithPropertyPathStrict(propertyPath);
    if (!column)
        throw new FindCriteriaNotFoundError_1.FindCriteriaNotFoundError(propertyPath, metadata);
    const aliasPath = `${alias}.${propertyPath}`;
    const parameterName = alias + '_' + propertyPath.replace('.', '_') + '_' + parameterIndex;
    const parameterValue = column.transformer && column.transformer.to ? column.transformer.to(fo) : fo;
    if (!(parameterValue instanceof typeorm_1.FindOperator))
        throw new FindCriteriaNotFoundError_1.FindCriteriaNotFoundError(propertyPath, metadata);
    const parameters = [];
    if (parameterValue.useParameter) {
        const realParameterValues = parameterValue.multipleParameters
            ? parameterValue.value
            : [parameterValue.value];
        for (let realParameterValueIndex = 0; realParameterValueIndex < realParameterValues.length; realParameterValueIndex++) {
            const realParameterValue = realParameterValues[realParameterValueIndex];
            if (typeof realParameterValue === 'number') {
                parameters.push(realParameterValue);
            }
            else {
                (root || query).expressionMap.nativeParameters[parameterName + realParameterValueIndex] = realParameterValue;
                parameterIndex++;
                parameters.push((root || query).connection.driver.createParameter(parameterName + realParameterValueIndex, parameterIndex - 1));
            }
        }
    }
    return parameterValue.toSql((subBuilder || query).connection, aliasPath, parameters);
};
const buildConditionForFilter = (entity, query, unprocessedFilter, root, subBuilder) => {
    const relations = query.connection.getMetadata(entity).relations.reduce((p, n) => ({
        ...p,
        [n.propertyName]: n
    }), {});
    const stripLogicalFilters = (filter) => {
        const result = [];
        for (const key of Object.keys(filter || {})) {
            const value = (filter || {})[key] || typeorm_1.IsNull();
            switch (key) {
                case 'AND': {
                    const values = Object.values(value);
                    if (!values.length)
                        continue;
                    result.push(Object.assign(new types_1.AndFilter(), values.map(stripLogicalFilters)));
                    break;
                }
                case 'OR': {
                    const values = Object.values(value);
                    if (!values.length)
                        continue;
                    result.push(Object.assign(new types_1.OrFilter(), values.map(stripLogicalFilters)));
                    break;
                }
                case 'NOT': {
                    const values = Object.values(value);
                    if (!values.length)
                        continue;
                    result.push(Object.assign(new types_1.NotFilter(), values.map(stripLogicalFilters)));
                    break;
                }
                default: {
                    const match = key.match(/^(?<path>[a-zA-Z]+)((_(?<logic>(some|every|none)))|)$/);
                    if (!match || !match.groups) {
                        result.push(new types_1.Filter({ key, value }));
                        break;
                    }
                    const { path, logic } = match.groups;
                    if (path in relations) {
                        result.push(new types_1.RelationFilter({
                            type: logic ? 'multiple' : 'single',
                            logic,
                            path,
                            from: new relations[path].target(),
                            value: Object.assign(new relations[path].inverseEntityMetadata.target(), value)
                        }));
                        break;
                    }
                    result.push(new types_1.Filter({ key, value }));
                    break;
                }
            }
        }
        return result;
    };
    const buildQueryConditions = (query, where, filter, logic = undefined) => {
        for (let i = 0; i < filter.length; i++) {
            const filterType = filter[i];
            switch (filterType.constructor) {
                case types_1.AndFilter:
                case types_1.OrFilter:
                case types_1.NotFilter: {
                    const brackets = (value, logic = undefined) => {
                        return new typeorm_1.Brackets(wb => buildQueryConditions(query, wb, value, logic));
                    };
                    if (filterType instanceof types_1.OrFilter) {
                        (where || query).andWhere(new typeorm_1.Brackets(qb => {
                            return Object.values(filterType || {}).forEach(value => {
                                qb.orWhere(brackets(value));
                            });
                        }));
                    }
                    else {
                        (where || query).andWhere(new typeorm_1.Brackets(qb => {
                            return Object.values(filterType || {}).forEach(value => {
                                qb.andWhere(brackets(value, filterType));
                            });
                        }));
                    }
                    break;
                }
                case types_1.RelationFilter: {
                    (where || query).andWhere(() => {
                        const subQuery = query.subQuery();
                        const pk = metadata_1.getMetadataStorage().getPk(filterType.from.constructor.name) ||
                            (() => {
                                const metadata = query.connection.getMetadata(filterType.from.constructor);
                                for (const column of metadata.primaryColumns) {
                                    if (column.target === metadata.target || typeof column.target !== 'function') {
                                        continue;
                                    }
                                    const pk = metadata_1.getMetadataStorage().getPk(column.target.name);
                                    if (pk) {
                                        return pk;
                                    }
                                }
                                return undefined;
                            })();
                        if (!pk)
                            throw new Error(`${filterType.from.constructor.name} must have decorator Id for Model.`);
                        const cloneAlias = filterType.from.constructor.name === filterType.value.constructor.name ? `_` : '';
                        const subQuerySql = subQuery
                            .select(`${filterType.from.constructor.name}.${pk}`)
                            .from(filterType.from.constructor, `${filterType.from.constructor.name}`)
                            .innerJoin(`${filterType.from.constructor.name}.${filterType.path}`, `${filterType.value.constructor.name}${cloneAlias}`);
                        buildConditionForFilter(filterType.value.constructor, subQuery, filterType.value, root || query, subQuery.createQueryBuilder().from(filterType.value.constructor, `${filterType.value.constructor.name}${cloneAlias}`));
                        switch (filterType.type) {
                            case 'multiple':
                                if (filterType.logic === 'none') {
                                    return `"${filterType.from.constructor.name}"."${pk}" NOT IN ${subQuerySql.getQuery()}`;
                                }
                                else {
                                    return `"${filterType.from.constructor.name}"."${pk}" IN ${subQuerySql.getQuery()}`;
                                }
                            case 'single':
                                return `"${filterType.from.constructor.name}"."${pk}" IN ${subQuerySql.getQuery()}`;
                            default:
                                return '';
                        }
                    });
                    break;
                }
                case types_1.Filter: {
                    const { key, value } = filterType;
                    const [attr, operator] = operators_1.getFindOperator(String(key), value);
                    const condition = buildWhere(query, attr, operator, undefined, root, subBuilder);
                    if (logic instanceof types_1.NotFilter) {
                        (where || query).andWhere(`NOT(${condition})`);
                    }
                    else {
                        (where || query).andWhere(condition);
                    }
                    break;
                }
                default:
                    throw new Error(`filterType: ${filterType} didn't recognize.`);
            }
        }
    };
    const filter = stripLogicalFilters(unprocessedFilter);
    buildQueryConditions(query, undefined, filter);
    return query;
};
const filter = (entity, { where }, queryBuilder) => {
    const query = queryBuilder || entity.createQueryBuilder();
    if (!where ||
        (() => {
            for (const prop in where || {}) {
                if (Object.prototype.hasOwnProperty.call(where, prop)) {
                    return true;
                }
            }
            return false;
        })()) {
        buildConditionForFilter(entity, query, where);
    }
    return query;
};
exports.default = filter;
